
/*M*                     
//                    INTEL CORPORATION PROPRIETARY INFORMATION
//      This software is supplied under the terms of a license agreement or
//      nondisclosure agreement with Intel Corporation and may not be copied
//      or disclosed except in accordance with the terms of that agreement.
//            Copyright (c) 1995 Intel Corporation.  All Rights Reserved.
//
*M*/
/////////////////////////////////////////////////////////
// Name:    Imb_Utils.c
// Module:	IMBDRV
// Purpose: Contains the Function implementation for the ImbUtilsGetCPUInformation Module
/////////////////////////////////////////////////////////

#include "imb_os.h" 
#include "imb_drv.h"
#include "smbios.h"
#include "hist.h"

#include "Imb_Utils.h"
//#include "..\pmal\SMBiosDefs.h"
#include "SMBiosDefs.h"

/////////////////////////////////////////////////////////////////////////////////////////////////
// ImbUtilsGetCPUInformation
/////////////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbUtilsGetCPUInformation
//  Purpose:    To build CPU - Slot position information from the SMBIOS.
//  Context:    public function
//  Returns:    an unsigned long variable which contains the CPU slot population
//  with each bit set true representing CPU presence in that slot.
*F*/

DWORD ImbUtilsGetCPUInformation
	(
		 PDEVICE_EXTENSION deviceExtension
	 )
{

	PSMBIOS_PROCESSOR_INFO	pSmbiosProcessorInfo;
	PSMBIOS_ENTRY_POINT_STRUCTURE pSmbiosEPS = NULL;
	PSMBIOS_HEADER pSmbiosHeader;

	PVOID pvMappedAddress;
	PHYSICAL_ADDRESS	paPhysicalAddress;
	DWORD dwCPUMap = 0;
	size_t stMappingLength;
	DWORD dwIndex;
	PUCHAR puchVirtualAddress;
	UCHAR ucChecksum = 0;

	KIRQL	oldIrql;

	unsigned int uiTableLength;
	unsigned long ulTableAddress;

	KeAcquireSpinLock (&deviceExtension->UtilsSpinLock, &oldIrql);

	do {

		paPhysicalAddress.HighPart = 0;
		paPhysicalAddress.LowPart = SMBIOS_MEM_RANGE_BASE;

		stMappingLength = SMBIOS_MAPPED_RANGE_LENGTH;
		pvMappedAddress = MmMapIoSpace (paPhysicalAddress, stMappingLength, MmNonCached);
		
		if (NULL == pvMappedAddress) {
			break;
		} 

		puchVirtualAddress = (PUCHAR) pvMappedAddress;

		// First locate the "_SM_" in the SMBIOS
		for (dwIndex = 0; dwIndex < (SMBIOS_MAPPED_RANGE_LENGTH - (sizeof (SMBIOS_ANCHOR_STRING) - 1) ) ; dwIndex++) {
			if (0 == memcmp ((PUCHAR)((ULONG_PTR)puchVirtualAddress + dwIndex), SMBIOS_ANCHOR_STRING, sizeof (SMBIOS_ANCHOR_STRING) - 1 ) ) {
				// SMBIOS anchor string located
				// relocate the pointer to the SMBIOS Structure Entry point
				pSmbiosEPS = (PSMBIOS_ENTRY_POINT_STRUCTURE) ((ULONG_PTR)puchVirtualAddress + dwIndex);
				break;
			}
		}

		// Check for the validity of the pointer
		if (NULL == pSmbiosEPS) {
			break;
		}

		
		// validate the BIOS checksum value.
		for (dwIndex = 0; dwIndex < (pSmbiosEPS->byLength); dwIndex++) {
			ucChecksum += (UCHAR) *((PUCHAR) pSmbiosEPS + dwIndex);
		}
		
		if (ucChecksum) {
			// Wrong checksum in the BIOS. just return
			break;
		}
				
			
		ulTableAddress = pSmbiosEPS->ulTableAddress;
		uiTableLength = pSmbiosEPS->wTableLength;

		// Free the memory, once we locate the SMBIOS table
		MmUnmapIoSpace (pvMappedAddress, stMappingLength);

		paPhysicalAddress.HighPart = 0;
		paPhysicalAddress.LowPart = ulTableAddress;

		stMappingLength = uiTableLength;
		pvMappedAddress = MmMapIoSpace (paPhysicalAddress, stMappingLength, MmNonCached);
		
		if (NULL == pvMappedAddress) {
			break;
		} 

		
		// locate the TYPE 4 Processor specific information
		puchVirtualAddress = (PUCHAR) pvMappedAddress;

		pSmbiosHeader = (PSMBIOS_HEADER) puchVirtualAddress;

		while (SMB_STRUCTURE_TYPE_PROCESSOR_INFO != pSmbiosHeader->Type) {
			
			// Travel to the end of the type structure
			puchVirtualAddress += pSmbiosHeader->Length;

			
			// Travel to the end of the descriptor strings
			while ( (puchVirtualAddress [0] != 0) || (puchVirtualAddress [1] != 0) ) {
				puchVirtualAddress++;
			}
			
			// Adjust the pointer for 2 '0's found
			puchVirtualAddress += 2;

			pSmbiosHeader = (PSMBIOS_HEADER) puchVirtualAddress;

		}

		pSmbiosProcessorInfo = (PSMBIOS_PROCESSOR_INFO) puchVirtualAddress;

		// For the total number of processors present, check the CPU socket populated flag
		// and build the CPU SLOT map.

		dwIndex = 0;

		while (SMB_STRUCTURE_TYPE_PROCESSOR_INFO == pSmbiosProcessorInfo->smBiosHeader.Type) {
			
			// Check for CPU Presence in the current slot
			if (SMB_PROC_INFO_CPU_SOCKET_POPULATED & pSmbiosProcessorInfo->Status) {
				dwCPUMap += (1 << dwIndex);
			}

			dwIndex++;

			// Travel to the end of the type structure
			puchVirtualAddress += pSmbiosProcessorInfo->smBiosHeader.Length;
			
			// Travel to the end of the descriptor strings
			while ( (puchVirtualAddress [0] != 0) || (puchVirtualAddress [1] != 0) ) {
				puchVirtualAddress++;
			}

			// Adjust the pointer for 2 '0's found
			puchVirtualAddress += 2;

			pSmbiosProcessorInfo = (PSMBIOS_PROCESSOR_INFO) puchVirtualAddress;
		}

		// Free the memory, once we locate the SMBIOS table
		MmUnmapIoSpace (pvMappedAddress, stMappingLength);

	} while (FALSE);

	KeReleaseSpinLock (&deviceExtension->UtilsSpinLock, oldIrql);

	return dwCPUMap;
}



/////////////////////////////////////////////////////////////////////////////////////////////////
// ImbUtilsGetSMBIOSUUIDInformation
/////////////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbUtilsGetSMBIOSUUIDInformation
//  Purpose:    To get the UUID info from the SMBIOS.
//  Context:    public function
//  Returns:    NTSTATUS
*F*/

NTSTATUS ImbUtilsGetSMBIOSUUIDInformation (
	PDEVICE_EXTENSION deviceExtension, 
	IN OUT PVOID IoBuffer
)
{
	NTSTATUS ntStatus;

	PSMBIOS_ENTRY_POINT_STRUCTURE pSmbiosEPS = NULL;
	PSMBIOS_HEADER pSmbiosHeader;

	PVOID pvMappedAddress;
	PHYSICAL_ADDRESS	paPhysicalAddress;
	size_t stMappingLength;
	DWORD dwIndex;
	PUCHAR puchVirtualAddress;
	UCHAR ucChecksum = 0;
	PSMBIOS_SYSTEM_INFORMATION pSMBIOSSystemInformation = NULL;

	KIRQL	oldIrql;

	unsigned int uiTableLength;
	unsigned long ulTableAddress;
	int i;
	UUID uuid, uuidTest;

	KeAcquireSpinLock (&deviceExtension->UtilsSpinLock, &oldIrql);

	do {
		paPhysicalAddress.HighPart = 0;
		paPhysicalAddress.LowPart = SMBIOS_MEM_RANGE_BASE;

		stMappingLength = SMBIOS_MAPPED_RANGE_LENGTH;
		pvMappedAddress = MmMapIoSpace (paPhysicalAddress, stMappingLength, MmNonCached);
		
		if (NULL == pvMappedAddress) {
            ntStatus = STATUS_INSUFFICIENT_RESOURCES;
			break;
		} 

		puchVirtualAddress = (PUCHAR) pvMappedAddress;

		// First locate the "_SM_" in the SMBIOS
		for (dwIndex = 0; dwIndex < (SMBIOS_MAPPED_RANGE_LENGTH - (sizeof (SMBIOS_ANCHOR_STRING) - 1) ) ; dwIndex++) {
			if (0 == memcmp ((PUCHAR)((ULONG_PTR)puchVirtualAddress + dwIndex), SMBIOS_ANCHOR_STRING, sizeof (SMBIOS_ANCHOR_STRING) - 1 ) ) {
				// SMBIOS anchor string located
				// relocate the pointer to the SMBIOS Structure Entry point
				pSmbiosEPS = (PSMBIOS_ENTRY_POINT_STRUCTURE) ((ULONG_PTR)puchVirtualAddress + dwIndex);
				break;
			}
		}

		// Check for the validity of the pointer
		if (NULL == pSmbiosEPS) {
			break;
		}

		// validate the BIOS checksum value.
		for (dwIndex = 0; dwIndex < (pSmbiosEPS->byLength); dwIndex++) {
			ucChecksum += (UCHAR) *((PUCHAR) pSmbiosEPS + dwIndex);
		}
		
		if (ucChecksum) {
            ntStatus = STATUS_UNSUCCESSFUL;
			// Wrong checksum in the BIOS. just return
			break;
		}
				
			
		ulTableAddress = pSmbiosEPS->ulTableAddress;
		uiTableLength = pSmbiosEPS->wTableLength;

		// Free the memory, once we locate the SMBIOS table
		MmUnmapIoSpace (pvMappedAddress, stMappingLength);

		paPhysicalAddress.HighPart = 0;
		paPhysicalAddress.LowPart = ulTableAddress;

		stMappingLength = uiTableLength;
		pvMappedAddress = MmMapIoSpace (paPhysicalAddress, stMappingLength, MmNonCached);
		

		if (NULL == pvMappedAddress) {
		    ntStatus = STATUS_INSUFFICIENT_RESOURCES;
			break;
		} 

		
		// locate the TYPE 1 System Info
		puchVirtualAddress = (PUCHAR) pvMappedAddress;

		pSmbiosHeader = (PSMBIOS_HEADER) puchVirtualAddress;

		while (SMB_STRUCTURE_TYPE_SYSTEM_INFO != pSmbiosHeader->Type) {
			
			// Travel to the end of the type structure
			puchVirtualAddress += pSmbiosHeader->Length;

			
			// Travel to the end of the descriptor strings
			while ( (puchVirtualAddress [0] != 0) || (puchVirtualAddress [1] != 0) ) {
				puchVirtualAddress++;
			}
			
			// Adjust the pointer for 2 '0's found
			puchVirtualAddress += 2;

			pSmbiosHeader = (PSMBIOS_HEADER) puchVirtualAddress;

		}

		pSMBIOSSystemInformation = (PSMBIOS_SYSTEM_INFORMATION) puchVirtualAddress;

		if (pSMBIOSSystemInformation) {
			uuid = pSMBIOSSystemInformation->Uuid;
			
			RtlCopyMemory (IoBuffer, &pSMBIOSSystemInformation->Uuid, sizeof (UUID) );
			ntStatus = STATUS_SUCCESS;
		}

	} while (FALSE);

	KeReleaseSpinLock (&deviceExtension->UtilsSpinLock, oldIrql);
	return ntStatus;
}
