
/*M*                     
//                    INTEL CORPORATION PROPRIETARY INFORMATION
//      This software is supplied under the terms of a license agreement or
//      nondisclosure agreement with Intel Corporation and may not be copied
//      or disclosed except in accordance with the terms of that agreement.
//            Copyright (c) 1995 Intel Corporation.  All Rights Reserved.
//
*M*/

#include "imb_os.h"
#include "smbios.h"
#include "debugprint.h"
#include "diagcodes.h"
#include "diagmsg.h"
#include "imb_drv.h"

extern ImbInterface *ImbInterfaces[];


/*F*
//*****************************************************************************
//* ScanForIPMIDeviceInfo(...)
//*****************************************************************************
//* Name:       ScanForIPMIDeviceInfo
//* Purpose:    Scans the SMBIOS table for IPMI Device Information record and
//*				returns the same.
//* Context:    Called by NT kernel at boot time.
//* Returns:    NTSTATUS, standard NT DDK defined status codes (STATUS_SUCCESS
//*             or error code)
//* Parameters:
//*				pDeviceExtension	 IN - Device Extension.
//*				paSmBiosTableAddress IN - Physical Address of SMBIOS table.
//*				ulSmBiosTableLength  IN - SMBIOS Table Length
//* Notes:      none
//*****************************************************************************
*F*/

NTSTATUS
ScanForIPMIDeviceInfo (
	IN	PDEVICE_EXTENSION	pDeviceExtension,
	IN	PHYSICAL_ADDRESS	paSmBiosTableAddress,
    IN  ULONG               ulSmBiosTableLength
	)
{
    int     i;
    ULONG   ulOffset = 0, ulOrgMemLength;
    PUCHAR  pOrgVirtualAddress = NULL, pVirtualAddress = NULL, pucTemp = NULL;
	KIRQL	currentIrql;
	NTSTATUS	ntStatus = STATUS_UNSUCCESSFUL;

    do
    {
		//
		// Check if the SMBIOS table address and length are valid...
		//
        if ((0x00 == paSmBiosTableAddress.LowPart) || \
            (ulSmBiosTableLength <= 0))
        {
            D_DiagMsg0(D_Errors, "Invalid Physical Address and Range of Bios Table.");
            break;
        }
		//
		// If the table address is not in 64K boundary - is usually isn't -
		// perform offset manipulation.
		//
		if (paSmBiosTableAddress.LowPart & BYTE_BOUNDARY_64K)
		{
            ulOffset = paSmBiosTableAddress.LowPart & BYTE_BOUNDARY_64K;
            paSmBiosTableAddress.LowPart &= ~((DWORD)BYTE_BOUNDARY_64K);
			ulSmBiosTableLength += ulOffset;
		}
		//
		// Map the physical address to virtual memory...
		//
        //ntStatus = MapPhysicalToVirtualMem(pDeviceExtension, paSmBiosTableAddress, &ulSmBiosTableLength, &pVirtualAddress);
        ulOrgMemLength = ulSmBiosTableLength;
		currentIrql = KeGetCurrentIrql();
        if (currentIrql != PASSIVE_LEVEL)
		{
			D_DiagMsg1(D_Errors,"IRQ level is too high: %x",currentIrql);
			break;
		}

        pVirtualAddress = pOrgVirtualAddress = MmMapIoSpace(paSmBiosTableAddress, ulSmBiosTableLength, MmNonCached);
        if (!pVirtualAddress)
		{
			D_DiagMsg0(D_Errors,"No Virtual Pointer to SMBios Table.");
			break;
		}
        pVirtualAddress += ulOffset;
		ulSmBiosTableLength -= ulOffset;
		while (ulSmBiosTableLength > 0)
		{
            //D_DiagMsg3(D_MidLevelInfo,"Searching Record # %x Length:%x, Remaining: %x", ((PSMBIOSHeader)pVirtualAddress)->Type, ((PSMBIOSHeader)pVirtualAddress)->Length, ulSmBiosTableLength);
            if (IPMI_DEVICE_INFORMATION == ((PSMBIOSHeader)pVirtualAddress)->Type)
			{
                D_DiagMsg0(D_MidLevelInfo, "Located IPMI Device Information Record...");
                switch(((PIPMIDeviceInformation) pVirtualAddress)->itInterfaceType)
                {
                    case    KCS:
                        D_DiagMsg0(D_InterfaceInfo,"KCS Interface detected. Setting up...");
                        pDeviceExtension->imbIf = ImbInterfaces[IMB_INTERFACE_KCS];
                        break;
                    case    SMIC:
                        D_DiagMsg0(D_InterfaceInfo,"SMIC Interface detected. Setting up...");
                        pDeviceExtension->imbIf = ImbInterfaces[IMB_INTERFACE_SMIC];
                        break;
                    case    Unknown:
                    case    BT:
                    default:
                        D_DiagMsg0(D_Errors,"BT/Unknown Interface NOT supported.");
                        break;
                }
                if (((PIPMIDeviceInformation) pVirtualAddress)->BaseAddress & IO_MAPPED_ADDRESS)
                {
                    D_DiagMsg0(D_InterfaceInfo,"Base Address is I/O Mapped.");
                    pDeviceExtension->bmcBaseAddress = \
                        (((PIPMIDeviceInformation) pVirtualAddress)->BaseAddress & ~((ULONGLONG)IO_MAPPED_ADDRESS));
                    pDeviceExtension->bmcBaseAddress |= (((PIPMIDeviceInformation) pVirtualAddress)->bBamLSBitAddress);
                    switch(((PIPMIDeviceInformation) pVirtualAddress)->bBamRegisterSpacing)
                    {
                        case    BYTE_BOUNDARY:
                            pDeviceExtension->bmcRegSpacing = 1;
                            break;
                        case    BIT32_BOUNDARY:
                            pDeviceExtension->bmcRegSpacing = 4;
                            break;
                        case    BYTE16_BOUNDARY:
                            pDeviceExtension->bmcRegSpacing = 16;
                            break;
                        default:
                            D_DiagMsg0(D_Errors,"Register Spacing Value is Invalid. Assuming default");
                            pDeviceExtension->bmcRegSpacing = 1;
                            break;
                    }
                }
                ntStatus = STATUS_SUCCESS;
				break;
			}
            ulSmBiosTableLength -= ((PSMBIOSHeader)pVirtualAddress)->Length;
            pVirtualAddress += ((PSMBIOSHeader)pVirtualAddress)->Length;
/*
			// IMPORTANT: This code to skip strings attached to the end of records
			// works only for 32-bit environment, referencing a pointer on 64-bit
			// without proper alignment, causes Kernel Fix-up to be called and found
			// to cause system freeze.
            for(i=0;(*((PUSHORT)((pVirtualAddress+i)) && ((ULONG)i < ulSmBiosTableLength));i++)
			{
                //scasb till double null or end of smbios table length.
			}
*/
			// Scan through the array of strings and reach the delimiter.
            for(i=0,pucTemp = pVirtualAddress;(ULONG)i < ulSmBiosTableLength;i++)
            {
                if (!(pucTemp[i] || pucTemp[i+1]))
                {
                    break;
                }
            }
			ulSmBiosTableLength -= (i+2);
            pVirtualAddress += (i+2);
			//Loop again till the end of smbios table is reached.
		}	// while
	}		
    while (FALSE);
    MmUnmapIoSpace(pOrgVirtualAddress, ulOrgMemLength);
    return ntStatus;
}

/*F*
//*****************************************************************************
//* GetIPMIDeviceInformation(...)
//*****************************************************************************
//* Name:       GetIPMIDeviceInformation
//* Purpose:    This function searches for SMBIOS entry point, obtains the
//*				table address and calls ScanForIpmiDeviceInfo to get Rec#38
//*				
//* Context:    Called by NT kernel at boot time.
//* Returns:    NTSTATUS, standard NT DDK defined status codes (STATUS_SUCCESS
//*             or error code)
//* Parameters:
//*             pDeviceExtension            OUT - Filled IPMI Device Information Record in Device Extn.
//* Notes:      none
//*****************************************************************************
*F*/
NTSTATUS
GetIPMIDeviceInformation (
	IN OUT	PDEVICE_EXTENSION	pDeviceExtension
	)
{
	NTSTATUS	ntStatus = STATUS_UNSUCCESSFUL;
	PHYSICAL_ADDRESS	paSmBiosAddress, paSmTableAddress;
    ULONG       ulLength, ulTableLength;
	PSMBIOSEntryPoint	pSmEp;
	UCHAR		ucChecksum;
    PUCHAR      pVirtualBaseAddress = NULL;
    ULONG       i;

	do
	{
        D_DiagMsg0(D_HighLevelInfo, "GetIPMIDeviceInformation: In");
		// Initialize the SMBIOS memory base address and range
		paSmBiosAddress.HighPart = 0L;
		paSmBiosAddress.LowPart = SMBIOS_MEM_RANGE_BASE;
		ulLength = SMBIOS_MAPPED_RANGE_LENGTH;

        //ntStatus = MapPhysicalToVirtualMem(pDeviceExtension, paSmBiosAddress, &ulLength, &pVirtualBaseAddress);
        pVirtualBaseAddress = MmMapIoSpace(paSmBiosAddress, ulLength, MmNonCached);
        if (!pVirtualBaseAddress)
		{
            ntStatus = STATUS_UNSUCCESSFUL;
            D_DiagMsg0(D_Errors,"No Virtual pointer to BIOS Memory.");
			break;
		}
        D_DiagMsg2(D_MidLevelInfo, "Virtual Pointer: %x, Length: %x Scanning Signature...", pVirtualBaseAddress, ulLength);
        for(i=0,pSmEp=NULL;i<ulLength - sizeof(SMBIOS_STRING) + 1;i++)
		{
			// Look for "_SM_" signature in the BIOS that defines the start of
			// entry point structure.
            if (sizeof(SMBIOS_STRING)-1 == RtlCompareMemory(pVirtualBaseAddress+i,SMBIOS_STRING,sizeof(SMBIOS_STRING)-1))
			{
                pSmEp = (PSMBIOSEntryPoint)(pVirtualBaseAddress+i);
                D_DiagMsg2(D_MidLevelInfo,"SMBIOS Entry Point @ %x, offset: %x", pSmEp, i);
				break;
			}
		}
		if (!pSmEp)
        {
            D_DiagMsg0(D_Errors,"Unable to Locate SMBIOS signature.");
			ntStatus = STATUS_UNSUCCESSFUL;
			break;
		}
		// Verify checksum of the structure...
		for(i=0,ucChecksum=0x00;i<pSmEp->byLength;i++)
		{
            ucChecksum += (UCHAR) *((PUCHAR)pSmEp + i);
		}
		if (ucChecksum)
		{
            D_DiagMsg0(D_Errors,"Invalid BIOS Entrypoint Checksum.");
			ntStatus = STATUS_IMAGE_CHECKSUM_MISMATCH;
			break;
		}
		// Check whether the version of the SMBIOS is compatible for
		// IPMI device information.
		if ((pSmEp->bySmBiosMajorVersion < SMBIOS_MAJOR_REVISION) || \
			(pSmEp->bySmBiosMinorVersion < SMBIOS_MINOR_REVISION))
		{
            D_DiagMsg2(D_Errors," SMBIOS Incompatible Version (%x.%x) detected.", \
                pSmEp->bySmBiosMajorVersion, pSmEp->bySmBiosMinorVersion);
			ntStatus = STATUS_REVISION_MISMATCH;
			break;
		}

        D_DiagMsg2(D_MidLevelInfo,"SMBIOS Table Address - %x,Length:%x",pSmEp->ulTableAddress,pSmEp->wTableLength);
		// Initialize the table address and length and call scanforipmideviceinfo
        paSmTableAddress.LowPart = pSmEp->ulTableAddress;
        paSmTableAddress.HighPart = 0L;
        ulTableLength = (ULONG) pSmEp->wTableLength;

        ntStatus = ScanForIPMIDeviceInfo(pDeviceExtension, paSmTableAddress, ulTableLength);
		if (!NT_SUCCESS(ntStatus))
		{
			D_DiagMsg0(D_Errors, "Record Type 38 Could NOT be located.");
			break;
		}
	}
	while (FALSE);
	// Unmap virtual memory when everything is done...
    MmUnmapIoSpace(pVirtualBaseAddress, ulLength);
    D_DiagMsg1(D_HighLevelInfo, "GetIPMIDeviceInformation: Out. Status: %x", ntStatus);
    return ntStatus;
}

